package org.jboss.jms.client;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class TestConsumer {

    public static void main(String[] args) throws Exception {

        if (args.length != 2) {
            System.out.println("Set parameters \"hostname\" and \"jndi_name_of_the_queue\"");
            System.exit(1);
        }

        Map<String, String> idChecked = new HashMap<String, String>();
        Context context = null;
        Connection con = null;
        Session session = null;

        try {

            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
            env.put(Context.PROVIDER_URL, "remote://" + args[0] + ":4447");

            context = new InitialContext(env);

            ConnectionFactory cf = (ConnectionFactory) (ConnectionFactory) context.lookup("jms/RemoteConnectionFactory");
            con = (Connection) cf.createConnection();

            con.start();

            session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            Queue queue = (Queue) context.lookup(args[1]);

            MessageConsumer consumer = session.createConsumer(queue);
            int counter = 0;
            Message message = null;
            Message lastMessage = null;
            
            while ((message = consumer.receive(3000)) != null) {

                counter++;
                System.out.println(counter + ". got message id = " + message.getJMSMessageID() + " message counter: "
                        + message.getStringProperty("count"));
                
                if (counter % 10 != 0) {
                        try {
                            message.acknowledge();
                        } catch (JMSException ex) {
                            Thread.sleep(10000);
                            message.acknowledge();
                            ex.printStackTrace();
                        }
                }
                Thread.sleep(10);
                lastMessage = message;
            }

            if (lastMessage != null)    {
                lastMessage.acknowledge();
            }
            System.out.println("Receiver ended - number of messages " + counter);

        } finally {
            if (session != null) {
                session.close();
            }
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (context != null) {
                try {
                    context.close();
                } catch (NamingException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
