package org.jboss.jms.client;

import java.util.Properties;
import java.util.UUID;
import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class TestProducer {

   
    private int numberOfRetries = 0;
    private static int maxRetries = 50;

    private Message createMessage(Session session, int counter) throws Exception {
        TextMessage message = session.createTextMessage(new String(new byte[1024]));
        message.setIntProperty("count", counter);
        
        return message;
    }

    public static void main(String[] args) {
        if (args.length != 3) {
            System.out.println("Set parameters \"hostname\" and \"jndi_name_of_the_queue\" ant \"number_of_messages\"");
            System.exit(1);
        }
        TestProducer testProducer = new TestProducer();
        testProducer.startClient(args);
    }

    public void startClient(String[] args) {

        final int MESSAGES_COUNT = Integer.valueOf(args[2]);

        Context context = null;
        Connection con = null;
        Session session = null;

        try {
            System.out.println("remote://" + args[0] + ":4447");
            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
            env.put(Context.PROVIDER_URL, "remote://" + args[0] + ":4447");
            
            context = new InitialContext(env);

            ConnectionFactory cf = (ConnectionFactory) context.lookup("jms/RemoteConnectionFactory");
            
            Queue queue = (Queue) context.lookup(args[1]);

            con = cf.createConnection();
            
            session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);

            MessageProducer producer = session.createProducer(queue);

            int i = 1;

            Message msg = null;

            while (i <= MESSAGES_COUNT) {
                
                msg = createMessage(session, i);
                
                msg.setStringProperty("_HQ_DUPL_ID", (UUID.randomUUID().toString() + System.currentTimeMillis()));
                
                try {
                    
                    producer.send(msg);
                
                    System.out.println("Sent message with property count: " + i + ", messageId:" + msg.getJMSMessageID());
                    
                            
                } catch (Exception ex) {
                    // exception can be thrown when something happens with server
                    resendMessage(producer, msg, i);
                    ex.printStackTrace();
                }
                
                i++;
            }

            producer.close();
            
        } catch (Exception e) {
            
            e.printStackTrace();
            
        } finally {

            if (session != null) {
                try {
                    session.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (context != null) {
                try {
                    context.close();
                } catch (NamingException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    
     private void resendMessage(MessageProducer producer, Message msg, int i)  {
        try {
            if (numberOfRetries > maxRetries) {
                try {
                    throw new Exception("Number of retries (" + numberOfRetries + ") is greater than limit (" + maxRetries + ").");
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            Thread.sleep(3000);
            numberOfRetries++;
            System.out.println("Sender RETRY (" + numberOfRetries + ") Sent message: " + i);
            producer.send(msg);
        } catch (JMSException ex) {
            ex.printStackTrace();
            resendMessage(producer, msg, i);
        } catch (InterruptedException ex) {
            ex.printStackTrace();
        }
        
    }

}
