package org.jboss.jms.client;

import java.security.Security;
import java.util.Properties;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.*;

public class TestProducerTransacted {

   
    private int numberOfRetries = 0;
    private static int maxRetries = 50;

    private Message createMessage(Session session, int counter) throws Exception {
        TextMessage message = session.createTextMessage(new String(new byte[1024]));
        message.setIntProperty("count", counter);
        
        return message;
    }

    public static void main(String[] args) {
        TestProducerTransacted testProducer = new TestProducerTransacted();
        testProducer.startClient();
    }

    public void startClient() {

        final int MESSAGES_COUNT = 1000000;

        Context context = null;
        Connection con = null;
        Session session = null;

        try {

            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
            env.put(Context.PROVIDER_URL, "remote://192.168.1.1:4447");
            
            context = new InitialContext(env);

            ConnectionFactory cf = (ConnectionFactory) context.lookup("jms/RemoteConnectionFactory");
            
            Queue queue = (Queue) context.lookup("jms/queue/testQueue0");

            con = cf.createConnection();
            
            session = con.createSession(true, Session.SESSION_TRANSACTED);

            MessageProducer producer = session.createProducer(queue);

            int i = 0;

            Message msg = null;

            while (i <= MESSAGES_COUNT) {
                
                msg = createMessage(session, i);
                
                try {
                    
                    producer.send(msg);
                            
                } catch (Exception ex) {
                    // exception can be thrown when something happens with server
                    resendMessage(producer, msg, i);
                    ex.printStackTrace();
                }
                
                System.out.println("Sent message with property count: " + i + ", messageId:" + msg.getJMSMessageID());
                
                i++;
                
                if (i % 1000 == 0)    {
                    session.commit();
                    System.out.println("COMMIT message with property count: " + i + ", messageId:" + msg.getJMSMessageID());
                }
                
            }
            
            session.commit();
            
            producer.close();
            
        } catch (Exception e) {
            
            e.printStackTrace();
            
        } finally {

            if (session != null) {
                try {
                    session.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (context != null) {
                try {
                    context.close();
                } catch (NamingException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    
     private void resendMessage(MessageProducer producer, Message msg, int i)  {
        try {
            if (numberOfRetries > maxRetries) {
                try {
                    throw new Exception("Number of retries (" + numberOfRetries + ") is greater than limit (" + maxRetries + ").");
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            Thread.sleep(3000);
            numberOfRetries++;
            System.out.println("Sender RETRY (" + numberOfRetries + ") Sent message: " + i);
            producer.send(msg);
        } catch (JMSException ex) {
            ex.printStackTrace();
            resendMessage(producer, msg, i);
        } catch (InterruptedException ex) {
            ex.printStackTrace();
        }
        
    }

}
